library(reshape2)
library(ggplot2)
simpleSIR=function(beta, alpha,N=300e6, I0=1000,  nstep=1000, step.size=1, eps=1, revert=nstep, revertbeta=beta){
  
  #setting up initial conditions
  S=N
I=I0
R=0
  out=matrix(nrow=0, ncol=4)
  colnames(out)=c("day","S","I","R")
  day=1
  for(s in 1:nstep){
    day=day+step.size
      I=I+(beta*S*I/N-alpha*I)*step.size
    S=S-(beta*S*I/N)*step.size
    R=R+(alpha*I)*step.size
out=rbind(out, c(day,S,I,R))
  if (s==revert)
      beta=revertbeta
      } 

  return(as.data.frame(out))
}  

#betaList is a list with two componenets of the same length beta and day. The first beta is assumed to correspond to day 1
TimeVariableSIR=function(betaList, alpha,N=300e6, I0=1000,  nstep=1000, step.size=1, eps=1){
  
  #setting up initial conditions
  S=N
  I=I0
  R=0
  out=matrix(nrow=0, ncol=5)
  colnames(out)=c("day","S","I","R", "beta")
  day=1
  beta=betaList$beta[1]
  nextBeta=betaList$beta[2]
  updateDay=betaList$day[2]
  curIndex=2
  for(s in 1:nstep){
    day=day+step.size
    if(day>updateDay && curIndex<=length(betaList$beta)){
      show(c(day, updateDay))
      stop("Something is wrong with betaList")
    }
    if(!is.na(updateDay)&&abs(day-updateDay)<step.size&&curIndex<=length(betaList$beta)){
    
      beta=nextBeta
    curIndex=curIndex+1
        nextBeta=betaList$beta[curIndex]
        updateDay=betaList$day[curIndex]
    }
    I=I+(beta*S*I/N-alpha*I)*step.size
    S=S-(beta*S*I/N)*step.size
    R=R+(alpha*I)*step.size
    out=rbind(out, c(day,S,I,R, beta))

  } 
  
  return(as.data.frame(out))
}  


plotTimeVariableSIR=function(out, max.step=NULL){
  if(!is.null(max.step)){
    out=out[1:max.step,]
  }
  out[,2:4]=out[,2:4]/300e6
  #rescale beta the right span
  mm=max(out[,-c(1,5)])
  out[,5]=out[,5]/3
  df=melt(out, id.vars = "day")
  
  ggplot(df)+geom_line(aes(x=day, y=value, color=variable), size=1.2)+theme_bw(base_size = 15)+ggtitle(paste("Final Recovered =", round((sum(out[nrow(out),"R"])),3)))+labs(y="relative value")+scale_color_discrete(name = "variable", labels = c("S", "I", "R", expression(beta)))+labs(x="time")+scale_y_continuous(    sec.axis = sec_axis( ~.*3, name=expression(beta)))
}



plotSimpleSIR=function(out, max.step=NULL){
if(!is.null(max.step)){
  out=out[1:max.step,]
}
  df=melt(out, id.vars = "day")
ggplot(df)+geom_line(aes(x=day, y=value, color=variable), size=1.2)+theme_bw(base_size = 15)  
}


CapStr = function(y) {
  c = strsplit(y, " ")[[1]]
  paste(toupper(substring(c, 1,1)), substring(c, 2),
        sep="", collapse=" ")
}

